/***************************************************
TASS Control Software by John Ackermann N8UR.  Based on Adafruit examples below
****************************************************/

char version[] = "v0.41 -- 4 Oct 2015";

// BOF preprocessor bug prevent - insert me on top of your arduino-code
// From: http://www.a-control.de/arduino-fehler/?lang=en
#if 1
__asm volatile ("nop");
#endif

// uncomment to enable Adafruit 2.8 inch TFT and capacitive touchscreen
#define USE_TS

// uncomment to enable Wiznet W5100 ethernet shield
#define USE_W5100

// comment this to set board up as single 8 pole using K9
// #define SPLIT

// uncomment this to display bitmasks on the serial console
#define SHOW_MASKS

#include "TASS_defines.h"

#ifdef USE_TS
  #include <Wire.h>
  #include <SPI.h>
  #include <Adafruit_GFX.h>
  #include <Adafruit_ILI9341.h>
  #include <Adafruit_FT6206.h>
  Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
  Adafruit_FT6206 ctp = Adafruit_FT6206();
#endif // USE_TS

#ifdef USE_W5100
  #include <SPI.h>
  #include <EthernetUdp.h>
  #include <Dhcp.h>
  #include <Dns.h>
  #include <util.h>
  #include <Ethernet.h>
  #include <EthernetServer.h>
  #include <EthernetClient.h>

// ******************************************************
// If using ethernet, set network stuff here:
  byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
  IPAddress ip(10,73,3,177);
  IPAddress gateway(10,73,1,1);
  IPAddress subnet(255,255,0,0);
  
  // what to echo to the telnet port, if anything
  // 0 = no echo, 1 = echo raw command, 2 = interpreted output
  int telnet_verbose = 2; 
//*******************************************************
  
  // variable
 EthernetServer server = EthernetServer(23);
 EthernetClient client = 0;
 
 #endif  // USE_W5100

char board = ' ';
int relay = 0;
char cmd = ' ';
byte relay_num = 0;
char relay_char = ' '; 
boolean newclient = 1;
boolean execute = 0;
boolean BadCmd = 0;

// these keep the relay settings for each board across commands
byte relay_mask_A = 0;
byte relay_mask_B = 0;
byte relay_mask_C = 0;
byte relay_mask_D = 0;

// these keep the hold settings for each board across commands
byte hold_mask_A = 0;
byte hold_mask_B = 0;
byte hold_mask_C = 0;
byte hold_mask_D = 0;

byte cmd_mask = 0;
byte relay_mask = 0;

// setup for serial input
const char startChar = '<';
const char endChar = '>';
char dataBuffer[DATABUFFERSIZE+1]; //Add 1 for NULL terminator

void setup(void) {
// set configuration pins to input pullup
  pinMode(14,INPUT_PULLUP);
  pinMode(15,INPUT_PULLUP);
  pinMode(16,INPUT_PULLUP);
  pinMode(17,INPUT_PULLUP);
  pinMode(18,INPUT_PULLUP);
  pinMode(19,INPUT_PULLUP);
  pinMode(22,INPUT_PULLUP);         
  
// initialize serial:
  Serial.begin(115200);
// splash
  Serial.println("TAPR TASS Control Software");
  Serial.println(version);
  Serial.println("Copyright 2015 John Ackermann   N8UR");
  Serial.println("Licensed under MIT license");
  Serial.println("");

// set up digital pins for each board
  for (int relay_num=A_1;relay_num<=A_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=B_1;relay_num<=B_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=C_1;relay_num<=C_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=D_1;relay_num<=D_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  
#ifdef USE_TS
  tft_setup();
#endif

#ifdef USE_W5100
  //disable SD
  pinMode(4,OUTPUT);
  digitalWrite(4,HIGH);

  Ethernet.begin(mac, ip, gateway, subnet);
  server.begin();    
#endif

} // setup

void loop() {
  // command bytes
  char board = ' ';
  int relay = 0;
  char cmd = ' ';
  boolean BadCmd = 0;

  if ( execute ) {
   
  }
  
  if (!execute) {
    #ifdef USE_TS 
      get_ts_data();
    #endif
    #ifdef USE_W5100
      get_w5100_data();
    #endif
    get_usb_data();
 }
 if (execute) {
    board = toupper(dataBuffer[1]);
    relay = atoi(&dataBuffer[2]);
    cmd = toupper(dataBuffer[3]); 
    
    byte cmd_mask = 0;
  
  

  // if S set the relay mask
    if (cmd == 'S') {
      // first, clear the relay mask; either appropriate half, or the whole thing  
      #ifdef SPLIT
      if (relay >=1 && relay <=4) { 
        relay_mask = clear_mask_top(relay_mask);
      } else {
        relay_mask = clear_mask_bottom(relay_mask);
      }   
     #else
     relay_mask = clear_mask(relay_mask);
    #endif  
    // now, set the mask
    relay_mask = set_mask(relay, relay_mask);
    }
    
    // if U clear appropriate hold mask
    
    if (cmd == 'U') {
      if (relay == 0) { 
        relay_mask = clear_mask(relay_mask);
      };
      switch (board) {
        case 'A': hold_mask_A = clear_mask(hold_mask_A); break;
        case 'B': hold_mask_B = clear_mask(hold_mask_B); break;
        case 'C': hold_mask_C = clear_mask(hold_mask_C); break;
        case 'D': hold_mask_D = clear_mask(hold_mask_D); break;
      }
   }

// if H set appropriate hold mask
    if (cmd == 'H') {
      switch (board) {
        case 'A':
          hold_mask_A = set_mask(relay,hold_mask_A);
          break;
        case 'B':
          hold_mask_B = set_mask(relay,hold_mask_B);
          break;
        case 'C':
          hold_mask_C = set_mask(relay,hold_mask_C);
          break;
        case 'D':
          hold_mask_D = set_mask(relay,hold_mask_D);
          break;
        } // switch
      } // if

// combine hold and relay to get command
    switch (board) {
    case 'A':
      write_board(A_1, hold_mask_A | relay_mask);
      break;
    case 'B':
      write_board(B_1, hold_mask_B | relay_mask);
      break;
    case 'C':
      write_board(C_1, hold_mask_C | relay_mask);
      break;
    case 'D':
      write_board(D_1, hold_mask_D | relay_mask);
      break;
    } // switch

  Serial.println("");
  Serial.print ("Board: ");
  Serial.print (board);
  Serial.print (" relay: ");
  Serial.print (relay);
  Serial.print (" command: ");
  Serial.print (cmd);
  if ((cmd == 'U') && (relay == 0)) { Serial.print(" -- Clear All"); }
  Serial.println();
  
  #ifdef SHOW_MASKS
    display_masks();
    Serial.println();
  #endif
  
  #ifdef USE_TS
  draw_status();
  #endif
  execute = 0;
 } // execute 
} // of loop
  
byte set_mask(int x, byte y) {
  switch (x) {
    case 1: y |= 0x01; break;
    case 2: y |= 0x02; break;
    case 3: y |= 0x04; break;
    case 4: y |= 0x08; break;
    case 5: y |= 0x10; break;
    case 6: y |= 0x20; break;
    case 7: y |= 0x40; break;
    case 8: y |= 0x80; break;
  }
  return y;
} // of set_mask

byte clear_mask_top(byte mask) {
  mask = mask & 0xF0;
     return mask;   
}

byte clear_mask_bottom(byte mask) {
  mask = mask & 0x0F;
  return mask;  
}

byte clear_mask(byte mask) { 
  mask = mask & 0x00;
  return mask;  
}

void write_board(int first_io, byte cmd_mask) {
    int counter = first_io; // first relay
    for (byte mask = 00000001; mask>0; mask <<= 1) { //iterate through bit mask
      if (cmd_mask & mask) { // if bitwise AND resolves to true
        digitalWrite(counter,HIGH);
      } else {
        digitalWrite(counter,LOW);
      } // of if/else
      counter++;
    } // of for

  #ifndef SPLIT
  // set K9
  if (relay >4) {
    digitalWrite(first_io+8,HIGH);
  } else {
    digitalWrite(first_io+8,LOW);
  } 
  #endif
}  // of write_board

void display_masks() {
   Serial.print("Relay Mask: ");
   printBits(relay_mask);
   Serial.print(" Hold Masks: ");
   printBits(hold_mask_A); 
   Serial.print(" ");
   printBits(hold_mask_B); 
   Serial.print(" ");
   printBits(hold_mask_C); 
   Serial.print(" ");
   printBits(hold_mask_D); 
   Serial.print("\n");
}
   
void printBits(byte myByte){
 for(byte mask = 0x80; mask; mask >>= 1){
   if(mask  & myByte)
       Serial.print('1');
   else
       Serial.print('0');
   }
}

